/*       -*- C++ -*- */
/** \file
********************************************************************
* $URL: svn+ssh://nalw02/home/alg/schlemme/svn/cextools/trunk/include/cexfile.h $
* $Id: cexfile.h 15 2010-06-25 19:53:27Z schlemme $
* $Date: 2010-06-25 21:53:27 +0200 (Fr, 25. Jun 2010) $
* $Revision: 15 $
* \author Tobias Schlemmer <keinstein@users.berlios.de>
* $Author: schlemme $ 
* \license LGPL
* 
* Main cexfile library include file
********************************************************************
* \dir cexfile
* The directory cexfile contains the internal class definitions for
* the .cex file
********************************************************************
* \defgroup cexparser .cex file parser
* The cex file parser can be used to parse a .cex file generated by
* Concept Explorer.
*
* It is implemented completely inline, so that no additional binary
* libraries are needed.  
*
*\{
********************************************************************
*/	  
#ifndef CEXFILE_H
#define CEXFILE_H

#include <libxml++/libxml++.h>
#include "cexfile/point.h"
#include "cexfile/label.h"
#include "cexfile/concept.h"
#include "cexfile/context.h"

#include "cexfile/lattice.h"

/// namespace of the cex file parser
namespace cexfile {

	/// A cex file parser class
	/***************************************************************/
	/**
	 * This class parses a given cex file using libxml++ Version 2. 
	 ***************************************************************/
	class cexfile {
    
		/** type of the cex file version */ 
		typedef std::pair<int,int> versiontype; 
		versiontype cexversion; ///< File version 
		Glib::ustring filename; ///< Name of the file to parse
		xmlpp::DomParser parser; ///< libxml++2 XML Parser
		context_s contexts; 
		///< List of the contexts in the \link cexfile::cexfile::filename .cex file \endlink
		lattice_s lattices; 
		///< List of the lattices in the \link  cexfile::cexfile::filename .cex file\endlink

	public:
		/// Parse the entire file.
		/** This function will parse the entire file associated with the
		    object. */
		cexfile& parse_file();

		/// Default constructor.
		/** This is the default constructor with no arguments. */
		cexfile():filename(),
			  cexversion(-1,-1),
			  parser(),
			  contexts()
			{}

		/// Standard constructor.
		/** This constructor loads the file given by \c n and calls
		    #parse_file() to initializes the object hierarcy from the
		    given XML file. 
		    \param n filename that shall be loaded.
		*/
		cexfile(Glib::ustring n):filename(n),
					 cexversion(-1,-1),
					 parser(),
					 contexts()
			{
				parse_file();
			}

		/// return lattices
		/** This function returns a constant reference to the lattices
		    owned by the object.
		    \return Reference to lattice_s */
		const lattice_s & get_lattices() { return lattices; }

		/** This function parses the cex file tree.
		    \param Node Root node of the cex XML data. */
		void parse_nodes(const xmlpp::Node * Node);
    
		/** This function parses the cex file version information.
		    \param Node Root node of the cex XML data. */
		void parse_version(const xmlpp::Node * Node);

		/** This function parses the contexts included in cex data.
		    \param Node Root node of the cex XML data. */
		void parse_contexts(const xmlpp::Node * Node);

		/** This function parses the recalculation policy of the cex data.
		    \param Node Root node of the cex XML data. */
		void parse_recalculation_policy(const xmlpp::Node * Node);

		/** This function parses the lattices and diagrams included in
		    the cex data.
		    \param Node Root node of the cex XML data. */
		void parse_lattices(const xmlpp::Node * Node);

		/** This function outputs some statistics to a given std::ostream.
		    \param o output stream object */
		std::ostream & statistics(std::ostream & o) const;
	};

	/// Stream output operator
	/** This operator outputs some statistics about a \c #cexfile object.
	    \param o output stream
	    \param f \c #cexfile object 
	    \relates cexfile::cexfile */
	inline std::ostream & operator<< (std::ostream & o, const cexfile & f) {
		return f.statistics(o);
	}

	inline cexfile & cexfile::parse_file() {

		//parser.set_validate();
		parser.set_substitute_entities();

		//We just want the text to be resolved/unescaped automatically.
		parser.parse_file(filename);

		if(parser) {
			//Walk the tree:
			const xmlpp::Node* pNode = parser.get_document()->get_root_node();
			//deleted by DomParser.
			parse_nodes(pNode);
		}
    
	}

	inline void cexfile::parse_nodes (const xmlpp::Node * Node) {
		if (!Node) throw xmlpp::validity_error("Empty parent node.");
		parse_version(Node);
		parse_contexts(Node);
		parse_recalculation_policy(Node);
		parse_lattices(Node);
	}

	inline void cexfile::parse_version (const xmlpp::Node * Node) {
		if (!Node) throw xmlpp::validity_error("Empty parent Node for version.");

		xmlpp::NodeSet v = Node->find("./Version");
		if (v.size() < 1) 
			throw xmlpp::validity_error("No version node found.");
    
		for (xmlpp::NodeSet::iterator i = v.begin(); i != v.end(); i++){
			const xmlpp::Element* nodeElement = 
				dynamic_cast<const xmlpp::Element*>(*i);

			if (nodeElement) {
				xmlpp::Attribute * Attr = nodeElement->get_attribute("MajorNumber");
				if (Attr) 
					cexversion.first = atoi((Attr -> get_value()).c_str());

				Attr = nodeElement->get_attribute("MinorNumber");
				if (Attr) 
					cexversion.second = atoi((Attr -> get_value()).c_str());
			};
	
		}
		if (cexversion.first < 1 || cexversion.second < 0) {
			throw xmlpp::validity_error("File has no suitable version.");
		}
	}

	inline void cexfile::parse_recalculation_policy (const xmlpp::Node * Node) {
		if (!Node) 
			throw xmlpp::validity_error
				("Empty parent Node for recalculation policy.");

		Glib::ustring s = 
			parse_single_node_attribute(Node, "./RecalculationPolicy", "Value");

	}

	inline void cexfile::parse_contexts (const xmlpp::Node * Node) {
		if (!Node) throw xmlpp::validity_error("Empty parent Node for contexts.");

		xmlpp::NodeSet c = Node->find("./Contexts/Context");

		if (c.size() < 1) 
			throw xmlpp::validity_error("No context node found.");
    
		for (xmlpp::NodeSet::iterator i = c.begin(); i != c.end(); i++){
			contexts += context(*i);
		}
	}

	inline void cexfile::parse_lattices (const xmlpp::Node * Node) {
		if (!Node) throw xmlpp::validity_error("Empty parent Node for lattices.");

		xmlpp::NodeSet s = Node->find("./Lattices/Lattice");

		for (xmlpp::NodeSet::iterator i = s.begin(); i != s.end(); i++){
			lattices += lattice(*i,contexts);
		}
	}

	inline std::ostream & cexfile::statistics(std::ostream & o) const {
		o << lattices.size() << " lattices:" << std::endl;
		int j = 1;
		for (lattice_s::const_iterator i=lattices.begin(); i!=lattices.end(); i++) {
			o << "lattice " << j++ << std::endl;
			o << *i << std::flush;
		}
		return o;
	}


}

#endif /* CEXFILE_H */

/** \} */
