/*       -*- C++ -*- */
/** \file
 ********************************************************************
 * $URL: svn+ssh://nalw02/home/alg/schlemme/svn/cextools/trunk/include/cexfile/point.h $
 * $Id: point.h 12 2008-02-26 15:45:35Z schlemme $
 * $Date: 2008-02-26 16:45:35 +0100 (Di, 26 Feb 2008) $
 * $Revision: 12 $
 * \author Tobias Schlemmer <keinstein@users.berlios.de>
 * $Author: schlemme $ 
 * \license LGPL
 * 
 * A point class.
 ********************************************************************
 */	  
#ifndef CEXFILE_POINT_H
#define CEXFILE_POINT_H
#include <glibmm/stringutils.h>
#include <utility>
#include <iostream>

#include "cexfile/constants.h"
#include "cexfile/utility.h"


namespace cexfile {
  
  /// a point in the plane
  /** \note points are used to describe any type of vectors. This
      violates the distinction between affine and linear spaces, but
      it works.
   */
  class point: std::pair<double, double> {
    /// corresponding XML node
    const xmlpp::Node * node;
  public:
    ///  default constructor; 
    point():node(NULL) {}

    /// standard constructor 
    /** \param Node XML node to be parsed 
     */
    point(const xmlpp::Node * Node);

    /// create a point from coordinates
    /** \param x first coordinate
	\param y second coordinate
    */
    point(first_type x, second_type y):std::pair<double,double>(x,y) {}

    /// get the first coordinate
    /** \return reference to the first coordinate 
     */
    const first_type & get_x() const { return first; }

    /// get the second coordinate
    /** \return reference to the second coordinate 
     */
    const second_type & get_y() const { return second; }

    /// sets the first coordinate
    /** \param x first coordinate
    */
    void set_x(first_type x) { first=x; }

    /// sets the second coordinate
    /**	\param y second coordinate
     */
    void set_y(first_type y) { second=y; }

    /// add a point 
    /** Treat the given point as vector and move the current one
	according to it.
	\param p point to be added
	\return self reference after adding
    */
    point & operator += (const point & p) {
      first += p.first;
      second += p.second;
      return *this;
    }

    /// add two points
    /** Treat the given point as vector and add it to the current one
	according to it. The current point is not changed.
	\param p point to be added
	\return new point containing the result
    */
    point operator + (const point & p) const {
      point retval = *this; 
      return retval += p;
    }

    /// subtract a point 
    /** Treat the given point as vector and move the current one
	according to its inverse.
	\param p point to be subtracted
	\return self reference after adding
    */
    point & operator -= (const point & p) {
      first -= p.first;
      second -= p.second;
      return *this;
    }

    /// subtracht two points
    /** Treat the given point as vector and add its inverse to the
	current one. The current point is not changed.
	\param p point to be subtracted
	\return new point containing the result
    */
    point operator - (const point & p) const {
      point retval = *this; 
      return retval -= p;
    }
  };

  /// output operator
  /** This operator outputs a point %object in the form "(x;y)" to the
      given stream.
      \relates point
      \param o output stream
      \param p point to be output
      \return reference to the output stream
  */
  inline std::ostream & operator << (std::ostream & o , const point & p) {
    o << "(" << p.get_x() << ";" << p.get_y() << ")" ; 
    return o;
  }

  inline point::point(const xmlpp::Node * Node):node(Node) {
    if (!Node) throw xmlpp::validity_error("Empty parent Node for diagrams.");
    
#ifdef DEBUG
    std::cerr << "node_attribute:" << Node->get_path() << std::endl;
#endif

    Glib::ustring path = FIGURE_COORDS + "/" + POINT2D;

    xmlpp::NodeSet s = Node->find(path);
    if (s.size() < 1) 
      throw xmlpp::validity_error("there must be a node "
				   + Node->get_path() + "/" + 
				   path + " containing coordinate attributes.");
    if (s.size() > 1) 
      throw xmlpp::validity_error("There are too much nodes in the path "
				  + Node->get_path() + "/" + path + ".");

    first = Glib::Ascii::strtod(parse_attribute(*(s.begin()), POINT2D_X));
    second = Glib::Ascii::strtod(parse_attribute(*(s.begin()), POINT2D_Y));
#ifdef DEBUG
    std::cerr << first << std::endl;
    std::cerr << second << std::endl;
#endif
  }
};
#endif /* CEXFILE_POINT_H */
