/*       -*- C++ -*- */
/** \file
 ********************************************************************
 * $URL: svn+ssh://nalw02/home/alg/schlemme/svn/cextools/trunk/include/cexfile/utility.h $
 * $Id: utility.h 13 2008-04-21 08:52:07Z schlemme $
 * $Date: 2008-04-21 10:52:07 +0200 (Mo, 21 Apr 2008) $
 * $Revision: 13 $
 * \author Tobias Schlemmer <keinstein@users.berlios.de>
 * $Author: schlemme $ 
 * \license LGPL
 * 
 * utility functions class.
 ********************************************************************
 */	  
#ifndef CEXFILE_UTILITY_H
#define CEXFILE_UTILITY_H

#include "cexfile/constants.h"
#include "libxml++/libxml++.h"


namespace cexfile {
#ifdef DEBUG
#define CEXDEBUG(data) std::cerr << "cex debug: " << data << std::endl
#else
#define CEXDEBUG(data) ((void)0)
#endif

#define CexElementCast(element) dynamic_cast<const xmlpp::Element *>(element);

  /// returns the value of a given attribute of a node
  /** \param Node node where the attribute should exist       
      \param attribute XML %Attribute to be found
      \param must_exist true if we shall raise an exception in case
      the attribute does not exist.
      \return String representing the value found.
  */
  inline Glib::ustring parse_attribute(const xmlpp::Node * Node, 
				Glib::ustring attribute = VALUE_ATTRIBUTE,
				bool must_exist=true) 
  {
#ifdef DEBUG
    std::cerr << "attribute:" << Node->get_path() << std::endl;
#endif

    const xmlpp::Element* nodeElement = CexElementCast(Node);

    if (nodeElement) {
      xmlpp::Attribute * Attr = nodeElement->get_attribute(attribute);
      if (Attr) return Attr->get_value();
    }
    if (must_exist) 
      throw xmlpp::validity_error
	("Attribute " + attribute + " must exist in Node " + 
	 Node->get_path() + ".");
    return "";
  }

  
  /// parse an %attribute on a given path
  /** \param Node node where we will start the search
      \param path search path (XPATH)
      \param attribute XML %Attribute to be found
      \param must_exist true if we shall raise an exception in case
      the attribute does not exist.
      \return String representing the value found.
  */
  inline Glib::ustring 
  parse_single_node_attribute (const xmlpp::Node * Node, 
			       Glib::ustring path, 
			       Glib::ustring attribute = VALUE_ATTRIBUTE,
			       bool must_exist = true)
  {

#ifdef DEBUG
    std::cerr << "node_attribute:" << Node->get_path() << std::endl;
#endif

    xmlpp::NodeSet s = Node->find(path);
    if (s.size() < 1 && must_exist) 
      throw xmlpp::validity_error("there must be a node "
				   + Node->get_path() + "/" + 
				   path + " containing the attribute " +
				   attribute + ".");
    if (s.size() > 1) 
      throw xmlpp::validity_error("There are too much nodes in the path "
				  + Node->get_path() + "/" + path + ".");
    return parse_attribute(*(s.begin()), attribute, must_exist);
  }

  /// finds the content of the Name node to a given node.
  /** \param Node node for which we want the name
      \return Name if found
  */
  inline Glib::ustring parse_name (const xmlpp::Node * Node) {
    if (!Node) 
      throw xmlpp::validity_error("Cannot find a name for an empty node");

#ifdef DEBUG
    std::cerr << "Name: " << Node -> get_path() << std::endl;
#endif

    xmlpp::NodeSet names = Node->find("Name");
    if (names.size() > 1) 
      throw 
	xmlpp::validity_error("More than one name given for current node.");
    else if (names.size() < 1) 
      throw xmlpp::validity_error("No Name tag found, where expected.");
      
    xmlpp::Node * name = *(names.begin());
#ifdef DEBUG
    std::cerr << "Name: " << name -> get_path() << std::endl;
#endif

    xmlpp::Node::NodeList Texts = name->get_children();
    if (Texts.size() < 1) 
      throw xmlpp::validity_error("No Name empty.");
    else if (Texts.size() > 1)
      throw 
	xmlpp::validity_error("Multiple text nodes for Name not supported.");
      
    name = *(Texts.begin());

    const xmlpp::TextNode * textNode = 
      dynamic_cast<const xmlpp::TextNode *>(name);

    if (textNode) 
      return textNode->get_content();
    throw xmlpp::validity_error("Unsupported Child node for name.");
    
  }

  /// convert a boolean string reprensentation to bool
  /** \param value string to be parsed
      \retval true if the string is either "TRUE", "T" or "1" in 
      case insensitive comparison.
  */
  inline bool stob(Glib::ustring value) {
    if (value.uppercase() == "TRUE" || 
	value.uppercase() == "T" || 
	value == "1") 
      return true;
    return false;
  }

};
#endif /* CEXFILE_UTILITY_H */
