/** \file
 ********************************************************************
 * $URL: svn+ssh://nalw02/home/alg/schlemme/svn/cextools/trunk/pdfplot.cpp $
 * $Id: pdfplot.cpp 15 2010-06-25 19:53:27Z schlemme $
 * $Date: 2010-06-25 21:53:27 +0200 (Fr, 25. Jun 2010) $
 * $Revision: 15 $
 * \author Tobias Schlemmer <keinstein@users.berlios.de>
 * $Author: schlemme $ 
 * \license LGPL
 * 
 * PDF Plotting class. 
 *
 * \addtogroup cairoplotter
 * \{
 ********************************************************************
 */	  

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "pdfplot.h"
#include <cmath>
#include <sstream>

//using namespace cexfile;

pdfplot::default_value_type pdfplot::default_values = {
  1.0l,
  Cairo::LINE_CAP_ROUND,
  false
};

bool pdfplot::create_surface(std::string name, double width, double height) 
{
	surface =
	Cairo::PdfSurface::create(name + ".pdf", width, height);
	
	context = Cairo::Context::create(surface);
}

void pdfplot::plot() {
#ifdef DEBUG
  std::cerr << "plotting file "<< std::flush << filebase + ".pdf" 
	    << "..." << std::endl;
#endif
  
	if (!multi_file_mode) {
		create_surface(filebase);
	}

	const cexfile::lattice_s & lattices = file->get_lattices();
	for (cexfile::lattice_s::const_iterator i = lattices.begin();
	     i != lattices.end() ; i++) {
		plot_lattice(*i);
	}
	//  surface->finish();
#ifdef DEBUG
	std::cerr << "plotting file finished" << std::endl;
#endif
}

void pdfplot::plot_lattice(const cexfile::lattice &l) {
#ifdef DEBUG
	std::cerr << "plotting Lattice..." << std::endl;
#endif
	const cexfile::diagram_s & diagrams = l.get_diagrams();
	for (cexfile::diagram_s::const_iterator i = diagrams.begin(); 
	     i != diagrams.end(); i++) {
		plot_diagram(*i);
	}
#ifdef DEBUG
	std::cerr << "plotting Lattice finished" << std::endl;
#endif
}

void pdfplot::plot_diagram(const cexfile::diagram &d) {
	
	page_number++;
	
	if (multi_file_mode) {
		std::ostringstream str;
		str << filebase << "-" << page_number;
		create_surface(str.str());
	}
	
	const cexfile::edge_s & edges = d.get_edges();
	const cexfile::concept_s & concepts = d.get_concepts();
	
	context->select_font_face(std::string("Arial"),
				  Cairo::FONT_SLANT_NORMAL, 
				  Cairo::FONT_WEIGHT_NORMAL);
	context->set_font_size(d.get_font_size()); 
	textlabels.clear();
	dimensions = 
	cexfile::rect(cexfile::point(std::numeric_limits<double>::infinity(),
				     std::numeric_limits<double>::infinity()), 
		      cexfile::point(-std::numeric_limits<double>::infinity(),
				     -std::numeric_limits<double>::infinity()));

	get_edge_dimensions(edges);
	get_concept_dimensions(concepts,d);
	get_label_dimensions(d);

	size = dimensions.get_end() - dimensions.get_start();
#ifdef DEBUG
	std::cerr << "setting dimensions" << dimensions.get_start() 
	<< "-" << dimensions.get_end() << std::endl;
#endif

	if (surface->get_type() == Cairo::SURFACE_TYPE_PDF) {
		Cairo::RefPtr<Cairo::PdfSurface> pdfsurface =
		pdfsurface.cast_static(surface);
		pdfsurface->set_size(size.get_x()+4,size.get_y()+4);
	} else 	if (surface->get_type() == Cairo::SURFACE_TYPE_PS) {
		Cairo::RefPtr<Cairo::PsSurface> pssurface =
		pssurface.cast_static(surface);
		pssurface->set_size(size.get_x()+4,size.get_y()+4);
	} else 	if (multi_file_mode) {
		surface->finish();
		std::ostringstream str;
		str << filebase << "-" << page_number;
		create_surface(str.str(),size.get_x()+4,size.get_y()+4);

		context->select_font_face(std::string("Arial"),
					  Cairo::FONT_SLANT_NORMAL, 
					  Cairo::FONT_WEIGHT_NORMAL);
		context->set_font_size(d.get_font_size()); 
	}
	
	surface->set_device_offset(2 - dimensions.get_start().get_x(),
				   2-dimensions.get_start().get_y());
	//surface->set_device_offset(50,-50);
	//surface->set_device_offset(1 - dimensions.get_start().get_x(),
	//			     -size.get_y()-dimensions.get_start().get_y()-1);
	//  context->set_identity_matrix();
	context->set_line_width(default_values.line_width);
	context->set_line_cap(default_values.line_cap);

#ifdef DEBUG
	std::cerr << "Dimensions: " << dimensions.get_start()
	<< "-" << dimensions.get_end() << std::endl;
	std::cerr << "Surface_dims " << size << std::endl; 
#endif

	plot_edges(edges);
	plot_label_edges(d.get_uplabels());
	plot_label_edges(d.get_downlabels());
	plot_concepts(concepts,d);
	plot_labels();
	//  plot_uplabels(d);
	
	if (multi_file_mode) {
		surface->finish();
	} else {
		context->show_page();
	}
}

void pdfplot::plot_edges(const cexfile::edge_s &e) {
  for (cexfile::edge_s::const_iterator i = e.begin(); i != e.end(); i++) {
    context->save();
    cexfile::point p = get_pos(i->get_start());
    context->move_to(p.get_x(),p.get_y());
    p = get_pos(i->get_end());
    context->line_to(p.get_x(),p.get_y());
    context->stroke();
    context->restore();
  }
}

void pdfplot::plot_label_edges(const cexfile::label_s &s) {
  for (cexfile::label_s::const_iterator i = s.begin(); i != s.end(); i++) {
    const cexfile::edge & e = i -> get_link();
    context->save();
    context->set_source_rgb(1,0,0);
    cexfile::point p = get_pos(e.get_start());
    context->move_to(p.get_x(),p.get_y());
    p = get_pos(e.get_end());
    context->line_to(p.get_x(),p.get_y());
    context->stroke();
    context->restore();
  }
}

void pdfplot::plot_concepts(const cexfile::concept_s &e,
			    const cexfile::diagram &d) {
  for (cexfile::concept_s::const_iterator i = e.begin(); i != e.end(); i++) {
    double radius = d.get_node_radius(*i);
#ifdef DEBUG
    std::cerr << "Plot concept with count: " 
	      << i->get_object_count() << std::endl;
#endif
    cexfile::point p=get_pos(i->get_position());
    context->save();
    context->arc(p.get_x(),
		 p.get_y(),
		 radius,
		 0,
		 2.0 * M_PI);   
    context->set_source_rgb(1,1,1);
    Cairo::Path * path = context->copy_path();
    context->fill();
    if (i->get_object_count()) {
      context->set_source_rgb(0,0,0);
      context->arc(p.get_x(), p.get_y(), radius, 0, M_PI);
      context->fill();
    }
    if (i->get_attribute_count() ) {
      context->set_source_rgb(0,0,1);
      context->arc(p.get_x(), p.get_y(), radius, M_PI, 2*M_PI);
      context->fill();
    }
    context->append_path(*path);
    delete path;
    context->set_source_rgb(0,0,0);
    context->stroke();
    context->restore();
  }
}


void pdfplot::get_edge_dimensions(const cexfile::edge_s &e) {
  for (cexfile::edge_s::const_iterator i = e.begin(); i != e.end(); i++) {
#ifdef DEBUG
    std::cerr << "Edge: " << i->get_start() 
	      << "->" << i->get_end() << std::endl;
#endif 
    update_dimensions(i->get_start());
    update_dimensions(i->get_end());
  }
#ifdef DEBUG
  std::cerr << "Dimensions: " << dimensions.get_start()
	    << "-"
	    << dimensions.get_end() << std::endl;
#endif
}

void pdfplot::get_concept_dimensions(const cexfile::concept_s &e,
				     const cexfile::diagram &d) {
  concept_maxcount = d.get_object_count();
  for (cexfile::concept_s::const_iterator i = e.begin(); i != e.end(); i++) {
#ifdef DEBUG
    std::cerr << "Concept: " << i -> get_position() << std::endl;
#endif 
    double radius = d.get_node_radius(*i);
    update_dimensions(i->get_position(),radius);
  }
#ifdef DEBUG
  std::cerr << "Dimensions: " << dimensions.get_start()
	    << "-" << dimensions.get_end() << std::endl;
#endif
}

void pdfplot::get_uplabel_dimensions(const cexfile::diagram & d) {
  const cexfile::label_s & labels = d.get_uplabels();
  for (cexfile::label_s::const_iterator i = labels.begin(); 
       i!= labels.end(); i++) {
    textlabels += textpane(i->get_strings(), i->get_position(), context);
    const cexfile::rect & r = textlabels.back().get_extents();
    update_dimensions(r.get_start());
    update_dimensions(r.get_end());
  }
}

void pdfplot::get_downlabel_dimensions(const cexfile::diagram & d) {
  const cexfile::label_s & labels = d.get_downlabels();
  for (cexfile::label_s::const_iterator i = labels.begin(); i!= labels.end(); i++) {
    textlabels += textpane(i->get_strings(), i->get_position(), context);
    const cexfile::rect & r = textlabels.back().get_extents();
    update_dimensions(r.get_start());
    update_dimensions(r.get_end());
  }
}

void pdfplot::get_label_dimensions(const cexfile::diagram & d) {
  get_uplabel_dimensions(d);
  get_downlabel_dimensions(d);
}

void pdfplot :: plot_labels() {
  for (textpane_s::iterator i = textlabels.begin(); 
       i != textlabels.end(); i++) {
    i -> plot(context);
  }
}


/** \} */
